@@ -109,6 +109,7 @@ namespace {
   bool deviceCollate = false;
   cups_page_header2_t header;
   ppd_file_t *ppd = 0;
+  char pageSizeRequested[64];
   unsigned int bitmapoffset[2];
   unsigned int popplerBitsPerPixel;
   unsigned int popplerNumColors;
@@ -466,6 +467,9 @@ static void parseOpts(int argc, char **argv)
     exit(1);
 #endif /* HAVE_CUPS_1_7 */
   }
+  strncpy(pageSizeRequested, header.cupsPageSizeName, 64);
+  fprintf(stderr, "DEBUG: Page size requested: %s\n",
+	  header.cupsPageSizeName);
 }
 
 static void parsePDFTOPDFComment(FILE *fp)
@@ -1650,7 +1654,9 @@ static void outPage(poppler::document *doc, int pageNo,
   double paperdimensions[2], /* Physical size of the paper */
     margins[4];	/* Physical margins of print */
   ppd_size_t *size;		/* Page size */
+  ppd_size_t *size_matched = NULL;
   double l, swap;
+  int imageable_area_fit = 0;
   int i;
 
   poppler::page *current_page =doc->create_page(pageNo-1);
@@ -1684,20 +1690,40 @@ static void outPage(poppler::document *doc, int pageNo,
   memset(paperdimensions, 0, sizeof(paperdimensions));
   memset(margins, 0, sizeof(margins));
   if (ppd) {
+    imageable_area_fit = 0;
+    size_matched = NULL;
     for (i = ppd->num_sizes, size = ppd->sizes;
 	 i > 0;
-	 i --, size ++) {
+	 i --, size ++)
       /* Skip page sizes which conflict with settings of the other options */
       /* TODO XXX */
       /* Find size of document's page under the PPD page sizes */
-      if (fabs(header.PageSize[1] - size->length) < 5.0 &&
-	  fabs(header.PageSize[0] - size->width) < 5.0)
-	break;
-    }
-    if (i > 0) {
+      if (fabs(header.PageSize[1] - size->length) / size->length < 0.01 &&
+	  fabs(header.PageSize[0] - size->width) / size->width < 0.01 &&
+	  (size_matched == NULL ||
+	   !strcasecmp(pageSizeRequested, size->name)))
+	size_matched = size;
+    if (size_matched == NULL)
+      /* Input page size does not fit any of the PPD's sizes, try to fit
+	 the input page size into the imageable areas of the PPD's sizes */
+      for (i = ppd->num_sizes, size = ppd->sizes;
+	   i > 0;
+	   i --, size ++)
+	if (fabs(header.PageSize[1] - size->top + size->bottom) /
+	    size->length < 0.01 &&
+	    fabs(header.PageSize[0] - size->right + size->left) /
+	    size->width < 0.01 &&
+	    (size_matched == NULL ||
+	     !strcasecmp(pageSizeRequested, size->name))) {
+	  fprintf(stderr, "DEBUG: Imageable area fit\n");
+	  imageable_area_fit = 1;
+	  size_matched = size;
+	}
+    if (size_matched) {
       /*
        * Standard size...
        */
+      size = size_matched;
       fprintf(stderr, "DEBUG: size = %s\n", size->name);
       paperdimensions[0] = size->width;
       paperdimensions[1] = size->length;
@@ -1714,17 +1740,37 @@ static void outPage(poppler::document *doc, int pageNo,
        * landscape orientation...
        */
 
+      imageable_area_fit = 0;
+      size_matched = 0;
       for (i = ppd->num_sizes, size = ppd->sizes;
 	   i > 0;
 	   i --, size ++)
-	if (fabs(header.PageSize[0] - size->length) < 5.0 &&
-	    fabs(header.PageSize[1] - size->width) < 5.0)
-	  break;
-
-      if (i > 0) {
+	if (fabs(header.PageSize[0] - size->length) / size->length < 0.01 &&
+	    fabs(header.PageSize[1] - size->width) / size->width < 0.01 &&
+	    (size_matched == NULL ||
+	     !strcasecmp(pageSizeRequested, size->name)))
+	  size_matched = size;
+      if (size_matched == NULL)
+	/* Input page size does not fit any of the PPD's sizes, try to fit
+	   the input page size into the imageable areas of the PPD's sizes */
+	for (i = ppd->num_sizes, size = ppd->sizes;
+	     i > 0;
+	     i --, size ++)
+	  if (fabs(header.PageSize[0] - size->top + size->bottom) /
+	      size->length < 0.01 &&
+	      fabs(header.PageSize[1] - size->right + size->left) /
+	      size->width < 0.01 &&
+	      (size_matched == NULL ||
+	       !strcasecmp(pageSizeRequested, size->name))) {
+	    fprintf(stderr, "DEBUG: Imageable area fit\n");
+	    imageable_area_fit = 1;
+	    size_matched = size;
+	  }
+      if (size_matched) {
 	/*
 	 * Standard size in landscape orientation...
 	 */
+	size = size_matched;
 	fprintf(stderr, "DEBUG: landscape size = %s\n", size->name);
 	paperdimensions[0] = size->width;
 	paperdimensions[1] = size->length;
@@ -1746,7 +1792,9 @@ static void outPage(poppler::document *doc, int pageNo,
 	if (pwgraster == 0)
 	  for (i = 0; i < 4; i ++)
 	    margins[i] = ppd->custom_margins[i];
-	header.cupsPageSizeName[0] = '\0';
+	snprintf(header.cupsPageSizeName, 64,
+		 "Custom.%dx%d",
+		 header.PageSize[0], header.PageSize[1]);
       }
     }
   } else {
@@ -1780,9 +1828,13 @@ static void outPage(poppler::document *doc, int pageNo,
     }
   }
 
-
-  bitmapoffset[0] = margins[0] / 72.0 * header.HWResolution[0];
-  bitmapoffset[1] = margins[3] / 72.0 * header.HWResolution[1];
+  if (imageable_area_fit == 0) {
+    bitmapoffset[0] = margins[0] / 72.0 * header.HWResolution[0];
+    bitmapoffset[1] = margins[3] / 72.0 * header.HWResolution[1];
+  } else {
+    bitmapoffset[0] = 0;
+    bitmapoffset[1] = 0;
+  }
 
   /* write page header */
   if (pwgraster == 0) {
